@@ -1,6 +1,6 @@
 /* file: numeric_table.h */
 /*******************************************************************************
-* Copyright 2014-2020 Intel Corporation
+* Copyright 2014-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -197,6 +197,21 @@ public:
         _nrows  = nRows;
     }
 
+    /**
+     *  Sets data pointer to use for in-place calculation
+     *  \param[in] ptr      Shared pointer to the buffer
+     *  \param[in] nColumns Number of columns
+     *  \param[in] nRows    Number of rows
+     */
+    inline void setSharedPtr(const services::SharedPtr<DataType> & ptr, size_t nColumns, size_t nRows)
+    {
+        _xBuffer.reset();
+        _hostSharedPtr.reset();
+        _ptr   = ptr;
+        _ncols = nColumns;
+        _nrows = nRows;
+    }
+
     /**
      *  Sets data buffer to the table
      *  \param[in] buffer Buffer object that contains the memory
@@ -278,9 +293,12 @@ public:
     {
         _colsOffset = columnIdx;
         _rowsOffset = rowIdx;
-        _rwFlag     = rwFlag;
 
-        _hostSharedPtr.reset(); // need to reallocate cached pointer when rwFlag is changed
+        if (_rwFlag != rwFlag)
+        {
+            _rwFlag = rwFlag;
+            _hostSharedPtr.reset(); // need to reallocate cached pointer when rwFlag is changed
+        }
     }
 
     /**
