@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,12 +21,10 @@
 #include "oneapi/dal/algo/jaccard/vertex_similarity_types.hpp"
 #include "oneapi/dal/vertex_similarity.hpp"
 
-namespace oneapi::dal::preview {
-namespace detail {
+namespace oneapi::dal::preview::detail {
 
 template <typename Descriptor, typename Graph>
 struct vertex_similarity_ops<Descriptor, Graph, jaccard::detail::tag>
         : jaccard::detail::vertex_similarity_ops<Descriptor, Graph> {};
 
-} // namespace detail
-} // namespace oneapi::dal::preview
+} // namespace oneapi::dal::preview::detail
