@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2021 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,5 +19,5 @@
 #include "oneapi/dal/detail/common.hpp"
 
 namespace oneapi::dal::preview::load_graph::detail {
-ONEDAL_EXPORT int daal_string_to_int(const char *nptr, char **endptr);
+ONEDAL_EXPORT std::int32_t daal_string_to_int(const char *nptr, char **endptr);
 } // namespace oneapi::dal::preview::load_graph::detail
